#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <util/delay.h>
#include <avr/interrupt.h>

#include "ST7735R_TFT.h"
#include "spi.h"

#define DO 0
#define RE 1
#define MI 2
#define FA 3
#define SOL 4
#define LA 5
#define SI 6
#define DO2 7


// vector cu frecventele notelor
int frecventa_nota[24] = {
	130, 146, 164, 174,  195, 220, 246, 261,
	261, 293, 329, 349, 391, 440, 493, 523,
	523, 587, 659, 698, 783, 880, 987, 1031
};

// sunt 3 octave: 0, 1, 2
int octave;

// modul de utilizare: 0, 1, 2, 3
int mode;

// salveaza notele inregistrate
int melodie_rec[100];

// pozitia ultimei note inregistrate
int ultima_nota = 0;

// redare nota
void nota_pian(int nota) {
	int perioada_nota = 1000000 / frecventa_nota[nota];
	int i;
	PORTD |= (1 << PD5);
	for (i = 0; i < perioada_nota / 2; ++i) {
		_delay_us(1);
	}

	PORTD &= ~(1 << PD5);
	for (i = 0; i < perioada_nota / 2; ++i) {
		_delay_us(1);
	}
}

// redare frecventa
void nota_rec(int frecventa) {
	int perioada_nota = 1000000 / frecventa;
	int i;
	PORTD |= (1 << PD5);
	for (i = 0; i < perioada_nota / 2; ++i) {
		_delay_us(1);
	}

	PORTD &= ~(1 << PD5);
	for (i = 0; i < perioada_nota / 2; ++i) {
		_delay_us(1);
	}
}

void init_piano(void) {

	// setez tastele pianului ca intrare
	DDRC &= ~(1 << PC0);
	DDRA &= ~(1 << PA7);
	DDRC &= ~(1 << PC2);
	DDRC &= ~(1 << PC3);
	DDRC &= ~(1 << PC4);
	DDRC &= ~(1 << PC5);
	DDRC &= ~(1 << PC6);
	DDRC &= ~(1 << PC7);

	// activez rezistentele de pull-up pentru tastele pianului
	PORTC |= (1 << PC0);
	PORTA |= (1 << PA7);
	PORTC |= (1 << PC2);
	PORTC |= (1 << PC3);
	PORTC |= (1 << PC4);
	PORTC |= (1 << PC5);
	PORTC |= (1 << PC6);
	PORTC |= (1 << PC7);

	// setez butonul pentru octava ca intrare
	DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);

	// setez buton redare melodie ca intrare
	DDRA &= ~(1 << PA1);
	PORTA |= (1 << PA1);

	// setez butonul pentru play mode
	DDRA &= ~(1 << PA6);
	PORTA |= (1 << PA6);

	// setez butonul pentru rec mode
	DDRA &= ~(1 << PA5);
	PORTA |= (1 << PA5);

	// setez butonul pentru sing mode
	DDRA &= ~(1 << PA4);
	PORTA |= (1 << PA4);

	// setez buzzerul ca iesire
	DDRD |= (1 << PD5);

	// opresc buzzerul
	PORTD &= ~(1 << PD5);

	// setez ledul ca iesire
	DDRD |= (1 << PD7);

	// pornesc ledul
	PORTD |= (1 << PD7);

	// initilizez octava
	octave = 1;
}


int play_mode(void) {
	
	// nota DO
	if ((PINA & (1 << PA7)) == 0) {
		nota_pian(DO + octave * 8);
		return frecventa_nota[DO+octave*8];
	}
	
	// nota RE
	if ((PINC & (1 << PC0)) == 0) {
		nota_pian(RE + octave * 8);
		return frecventa_nota[RE+octave*8];
	}
	
	// MI
	if ((PINC & (1 << PC2)) == 0) {
		nota_pian(MI + octave * 8);
		return frecventa_nota[MI+octave*8];	
	}
	
	// FA
	if ((PINC & (1 << PC3)) == 0) {
		nota_pian(FA + octave * 8);
		return frecventa_nota[FA+octave*8];
	}
	
	// SOL
	if ((PINC & (1 << PC4)) == 0) {
		nota_pian(SOL + octave * 8);
		return frecventa_nota[SOL+octave*8];
	}
	
	// LA
	if ((PINC & (1 << PC5)) == 0) {
		nota_pian(LA + octave * 8);
		return frecventa_nota[LA+octave*8];
	}
	
	// SI
	if ((PINC & (1 << PC6)) == 0) {
		nota_pian(SI + octave * 8);
		return frecventa_nota[SI+octave*8];
	}
	
	// DO2
	if ((PINC & (1 << PC7)) == 0) {
		nota_pian(DO2 + octave * 8);
		return frecventa_nota[DO2+octave*8];
	}
	return 0;
}

void rec_mode(void){
	int frecventa = play_mode();	
	melodie_rec[ultima_nota++] = frecventa;
}

void sing_mode(void){
	int i, j;

	// reda notele din vector e rand
	for (i = 0; i < ultima_nota; i++) {
		//int n = notes[i] - '0'; n--;
		//int d = noteDuration[i] - '0';
		for (j = 0; j < 300; j++){
			nota_rec(melodie_rec[i]);
		}
		_delay_us(100);
	}

	// modific starea cu una inexistenta pentru a se opri redarea
	mode = 4;
}

void twinkle() {
	int i, j;
	int note[48] = {DO, DO, SOL, SOL, LA, LA, SOL,  0,
					 FA, FA, MI, MI, RE, RE, DO, 0,
					 SOL, SOL, FA, FA, MI, MI, RE, 0,
					 SOL, SOL, FA, FA, MI, MI, RE, 0,
					 DO, DO, SOL, SOL, LA, LA, SOL, 0,
					 FA, FA, MI, MI, RE, RE, DO, 0 };

	for (i = 0; i < 6; i++) {
		for (j = 0; j < 100; j++) {
			nota_pian(note[i]+8);
		}		
		_delay_us(2000);
	}
	for (i = 0; i < 200; i++) {
		nota_pian(note[6]+8);
	}
	_delay_us(100000);

	for (i = 8; i < 14; i++) {
		for (j = 0; j < 100; j++) {
			nota_pian(note[i]+8);
		}		
		_delay_us(2000);
	}
	for (i = 0; i < 200; i++) {
		nota_pian(note[14]+8);
	}
	_delay_us(100000);
	for (i = 16; i < 22; i++) {
		for (j = 0; j < 100; j++) {
			nota_pian(note[i]+8);
		}		
		_delay_us(2000);
	}
	for (i = 0; i < 150; i++) {
		nota_pian(note[22]+8);
	}
	_delay_us(100000);
	for (i = 24; i < 30; i++) {
		for (j = 0; j < 100; j++) {
			nota_pian(note[i]+8);
		}		
		_delay_us(2000);
	}
	for (i = 0; i < 150; i++) {
		nota_pian(note[30]+8);
	}
	_delay_us(100000);
	for (i = 32; i < 38; i++) {
		for (j = 0; j < 100; j++) {
			nota_pian(note[i]+8);
		}		
		_delay_us(2000);
	}
	for (i = 0; i < 150; i++) {
		nota_pian(note[38]+8);
	}
	_delay_us(100000);
	for (i = 40; i < 46; i++) {
		for (j = 0; j < 100; j++) {
			nota_pian(note[i]+8);
		}		
		_delay_us(2000);
	}
	for (i = 0; i < 150; i++) {
		nota_pian(note[46]+8);
	}

	// modific starea cu una inexistenta pentru a se opri redarea
	mode = 4;
}
void choose_mode(void){
	if ((PINA & (1 << PA6)) == 0) {
		ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
		ST7735R_DrawText(10, 50, "Let's sing a song!", 255, 255, 255, 0, 0, 0);
		mode = 0;
	}
	if ((PINA & (1 << PA5)) == 0) {
		ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
		ST7735R_DrawText(45, 50, "Record your", 255, 255, 255, 0, 0, 0);
		ST7735R_DrawText(35, 65, "masterpiece!", 255, 255, 255, 0, 0, 0);
		ultima_nota = 0;
		mode = 1;
		
	}
	if ((PINA & (1 << PA4)) == 0) {
		ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
		ST7735R_DrawText(30, 50, "Let's listen to ", 255, 255, 255, 0, 0, 0);
		ST7735R_DrawText(45, 65, "this HIT!", 255, 255, 255, 0, 0, 0);
		mode = 2;		
	}
	if ((PINA & (1 << PA1)) == 0) {
		ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
		ST7735R_DrawText(30, 50, "Twinkle Twinkle", 255, 255, 255, 0, 0, 0);
		mode = 3;		
	}
}

int main()
{
	init_piano();

    SPI_init();

    ST7735R_Begin();
    // curata ecranul
    ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);

	while(1){
		choose_mode();
		// selectez octava
		if ((PINA & (1 << PA0)) == 0) {
			octave = (octave + 1) % 3;
		}
		if (mode == 0) {
			play_mode();
		}
		if (mode == 1) {
			rec_mode();
		}
		if (mode == 2) {
			sing_mode();
		}
		if (mode == 3) {
			twinkle();
		}
	}	

    return 0;
}